import {
    Http, Metadata, Config, IEventParam,
} from '@zeedhi/core';
import {
    DialogService, Form, LoadingService, Modal, ModalService, Select, TextInput, Number, Text, FileInput, Button, Tabs, IGridEvent,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class ModeloDocumentosCdaController {
    private basicModal!: Modal;

    public tekgridVisivel: boolean = true;

    public currentRow: any = {};

    public rowDeleteAnexo: any = {};

    public habilitarBotao = true;

    get textFrame() {
        return this.currentRow.DSMODELODOCORCA;
    }

    get formVisivel() {
        return !this.tekgridVisivel;
    }

    public abrirFrame({ row }: any) {
        this.tekgridVisivel = false;
        this.currentRow = row;
        const tabs = Metadata.getInstance<Tabs>('tabsInformation-cda');
        tabs.activeTab = 0;
        this.carregarTabs();
    }

    public carregarTabs() {
        const tabs = Metadata.getInstance<Tabs>('tabsInformation-cda');
        const tabAtiva = tabs.activeTab;

        if (tabs.tabs[tabAtiva].name === 'tab_modelodocumentoCda') {
            this.carregarDadosAbaModeloDocumento();
        }

        if (tabs.tabs[tabAtiva].name === 'tab_anexosCda') {
            this.carregarDadosAbaAnexo();
        }
    }

    public carregarDadosAbaModeloDocumento() {
        Metadata.getInstance<TextInput>('DSMODELODOCORCA').value = this.currentRow.DSMODELODOCORCA;
        Metadata.getInstance<Select>('IDATIVO').value = this.currentRow.IDATIVO;
        Metadata.getInstance<Select>('IDDOCPADRAO').value = this.currentRow.IDDOCPADRAO;
    }

    public carregarDadosAbaAnexo() {
        const nrmodelodocda = this.currentRow.NRMODELODOCCDA;
        setTimeout(() => {
            const grid = Metadata.getInstance<TekGrid>('tab-anexos-cda');
            grid.datasource.setFilter({
                NRMODELODOCCDA: nrmodelodocda,
            });
        }, 1000);
    }

    public voltarTekGrid() {
        this.tekgridVisivel = true;
    }

    public async exibirModal() {
        this.basicModal = await ModalService.createFromJson('sm_cda/Modal/insercao_documento_cda', true);
        this.basicModal.show();
    }

    public fecharModal() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public async abrirModal() {
        await this.exibirModal();
        Metadata.getInstance<Text>('titleText').text = 'Novo Modelo de Documento';
    }

    public async exibirModalDocumento() {
        this.basicModal = await ModalService.createFromJson('sm_cda/Modal/insercao_idioma_documento_cda', true);
        this.basicModal.show();
    }

    public fecharModalDocumento() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public async abrirModalDocumento() {
        await this.exibirModalDocumento();
        Metadata.getInstance<Text>('titleText').text = 'Novo Anexo';
    }

    public async abrirModalEdicaoDocumento({ row }: any) {
        await this.exibirModalDocumento();
        Metadata.getInstance<Text>('titleText').text = 'Edição de Anexo';
        Metadata.getInstance<TextInput>('NRMODELODOCIDIOMA').value = row.NRMODELODOCIDIOMA;
        Metadata.getInstance<TextInput>('NRSEQMODDOCFCT').value = row.NRSEQMODDOCFCT;
        Metadata.getInstance<Select>('CDIDIOM').value = row.CDIDIOM;
        Metadata.getInstance<TextInput>('IDPADRAO_OLD').value = row.IDPADRAO;
        Metadata.getInstance<Select>('modalIdioma-IDATIVO').value = row.IDATIVO;
        Metadata.getInstance<TextInput>('NMARQUIVO').value = row.NMARQUIVO;
        Metadata.getInstance<TextInput>('DSCAMINHOARQ').value = row.DSCAMINHOARQ;
        Metadata.getInstance<TextInput>('DSTIPO').value = row.DSTIPO;
        Metadata.getInstance<Number>('VRTAMANHO').value = row.VRTAMANHO;

        if (Metadata.getInstance<TextInput>('NMARQUIVO').value != null) {
            Metadata.getInstance<FileInput>('ARQUIVO').isVisible = false;
        } else {
            Metadata.getInstance<FileInput>('ARQUIVO').isVisible = true;
        }
    }

    public showErrorDialogValidacao() {
        DialogService.show({
            name: 'error-dialog-validacao',
            type: 'error',
            title: 'Atenção',
            text: 'Não é possível adicionar outro registro do mesmo idioma para o mesmo modelo de documento',
        });
    }

    public async inserirModeloDocumentoIdioma() {
        const form = Metadata.getInstance<Form>('form-idioma-cda');
        const grid = Metadata.getInstance<TekGrid>('tab-anexos-cda');
        console.warn(form);
        if (Utilities.validateForm(form)) {
            const file = await Utilities.buildArrayToUpload('ARQUIVO');
            LoadingService.show();
            const params = {
                NRMODELODOCCDAIDI: Metadata.getInstance<TextInput>('FORM-IDI-CDA-NRMODELODOCCDAIDI').value,
                NRMODELODOCCDA: this.currentRow.NRMODELODOCCDA,
                CDIDIOM: Metadata.getInstance<Select>('FORM-IDI-CDA-CDIDIOM').value,
                IDPADRAO: Metadata.getInstance<Select>('FORM-IDI-CDA-CDIDIOM').selectValue.IDPADRAO,
                IDPADRAO_OLD: Metadata.getInstance<TextInput>('IDPADRAO_OLD').value,
                IDATIVO: Metadata.getInstance<Select>('modalIdioma-IDATIVO').value,
                NMARQUIVO: Metadata.getInstance<TextInput>('NMARQUIVO').value,
                DSCAMINHOARQ: Metadata.getInstance<TextInput>('DSCAMINHOARQ').value,
                DSTIPO: Metadata.getInstance<TextInput>('DSTIPO').value,
                VRTAMANHO: Metadata.getInstance<Number>('VRTAMANHO').value,
                ARQUIVO: file,
            };
            console.warn(params);
            Http.post('/inserirModeloDocumentoIdiomaCda', params).then(() => {
                this.fecharModalDocumento();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                if (error.response.data.error === 'EXISTE') {
                    this.showErrorDialogValidacao();
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public inserirModeloDocumentoCda() {
        const form = Metadata.getInstance<Form>('form-documento-cda');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-modelo-documento-cda');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/inserirModeloDocumentoCda', {
                NRMODELODOCCDA: Metadata.getInstance<TextInput>('FORM-DOC-NRMODELODOCCDA').value,
                DSMODELODOCORCA: Metadata.getInstance<TextInput>('FORM-DOC-DSMODELODOCORCA').value,
                IDATIVO: Metadata.getInstance<Select>('FORM-DOC-IDATIVO').value,
                IDDOCPADRAO: Metadata.getInstance<Select>('FORM-DOC-IDDOCPADRAO').value,
            }).then(() => {
                this.fecharModal();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public saveTabModeloDocumentoCda({ event }: IGridEvent) {
        event!.preventDefault();
        const form = Metadata.getInstance<Form>('form-modelo-documento-cda');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-modelo-documento-cda');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveTabModeloDocumentoCda', {
                NRMODELODOCCDA: Metadata.getInstance<TextInput>('NRMODELODOCCDA').value = this.currentRow.NRMODELODOCCDA,
                DSMODELODOCORCA: Metadata.getInstance<TextInput>('DSMODELODOCORCA').value,
                IDDOCPADRAO: Metadata.getInstance<Select>('IDDOCPADRAO').value,
                IDATIVO: Metadata.getInstance<Select>('IDATIVO').value,
            }).then(() => {
                grid.reload();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public dialogDeletarModeloDocumento({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        DialogService.show({
            name: 'dialog-deletar-modelo-doc',
            type: 'warning',
            title: 'Atenção',
            text: 'Todos os dados deste(s) modelo(s) de documento(s) serão perdidos. Deseja continuar?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarModeloDocumentoCda(component);
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deletarModeloDocumentoCda(component: TekGrid) {
        const NRS = (component.selectedRows.map((element: any) => element.NRMODELODOCCDA));
        LoadingService.show();
        Http.post('/deletarModeloDocumentoCda', {
            NRS,
        }).then(() => {
            component.selectedRows = [];
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public dialogDeletarAnexo({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        DialogService.show({
            name: 'dialog-deletar-anexo-idioma',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente excluir todos os registros selecionados? Seu documento anexado será perdido.',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarAnexo();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deletarAnexo() {
        const grid = Metadata.getInstance<TekGrid>('tab-anexos-cda');
        LoadingService.show();
        Http.post('/deletarAnexoCda', grid.selectedRows).then(() => {
            grid.selectedRows = [];
            grid.cancelEditedRows();
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public downloadAnexoModeloDocumentoCda({ row }: any) {
        const nrmodelodoccdaidi = row.NRMODELODOCCDAIDI;
        window.open(`${Config.endPoint}downloadAnexoModeloDocumentoCda?nrmodelodoccdaidi=${nrmodelodoccdaidi}`);
    }

    public dialogDeletarAnexoModeloDocumento({ row }: IEventParam<Button>) {
        this.rowDeleteAnexo = row;
        DialogService.show({
            name: 'dialog-deletar-anexo',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente excluir o documento anexado?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarAnexoModeloDocumento();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deletarAnexoModeloDocumento() {
        const grid = Metadata.getInstance<TekGrid>('tab-anexos-cda');
        LoadingService.show();
        Http.post('/deletarAnexoModeloDocumentoCda', {
            row: this.rowDeleteAnexo,
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public cliqueCelula({ column }: any) {
        let preventRowClick = false;
        if (column.name === 'DELETARANEXO' || column.name === 'DOWNLOADANEXO') {
            preventRowClick = true;
        }
        return preventRowClick;
    }

    public habilitarBotaoGrid({ row }: any) {
        return (row.NMARQUIVO != null);
    }
}
